;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; This is an attempt to create an adaptive derivative approximation
;;  scheme using the centered-difference formula to create a
;;  table of data. I'm basing this mostly off the adaptive
;;  Simpson's Lisp function that Dr. Long wrote to illustrate
;;  section 4.6 in our Mat360 class.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun adaptive-centered-recursive (f a b
				      &key
				      (epsilon 1e-4)
				      (tolerance 1e-6)
				      previous
				      )
  "
;; Examples:
 (func f(x) exp(-x^2/2))
 (def results (adaptive-centered-recursive #'f 0 3 :epsilon 1e-2 :tolerance 1e-4))
"
  (let* (
	 (xmean (mean (list a b)))
	 (int-l (centered-difference f a xmean))
	 (int-r (centered-difference f xmean b))
	 ;; after the first trial, the "grosser" derivative (previous) will
	 ;; be passed in (why recalculate?), however, if not, then we need
	 ;; to calculate it:
	 (previous (if previous previous (centered-difference f a b)))
	 (errors (abs (- previous (list int-l int-r))))
	 (new-epsilon (/ epsilon 2))
	 results
	 )
    
    ;; If our tolerance is met on either half, stop cutting:
    (if (> epsilon tolerance)
	(let ()
	  (setq results 
		(append results 
			(if (> (first errors) new-epsilon)
			    (adaptive-centered-recursive f a xmean
							 :epsilon new-epsilon
							 :tolerance tolerance
							 :previous int-l
							 )
			  (list (list (mean (list a xmean)) int-l)))))
	  (setq results 
		(append results 
			(if (> (second errors) new-epsilon)
			    (adaptive-centered-recursive f xmean b
							 :epsilon new-epsilon
							 :tolerance tolerance
							 :previous int-r
							 )
			  (list (list (mean (list xmean b)) int-r)))))
	  )
      (setq results 
	    (append results 
		    (list (list (mean (list a b)) previous))))
      )
    results
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;            Centered-Difference:
;;
;; The centered-difference formula on a single panel
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun centered-difference (f a b)
  (/ 
   (-
    (funcall f b)
    (funcall f a)
    )
   (- b a)
   )
  )

;;Let's go...
;; Here's our function:
(func f(x) exp(-x^2/2))

;; initialize our global for plotting (the boundaries of each panel):

;; run the code:
(setq
 results (adaptive-centered-recursive #'f 0 5 :epsilon 1e-2 :tolerance 1e-4)
 xs (mapcar #'first results)
 fprimes (mapcar #'second results)
 sorted-xs (sort-data xs)
 fs (mapcar #'f sorted-xs)
 done t
 )

;; Now plot it...
(xgobips 
 "AdaptiveDerivatives.ps"
 0
 0
 1
 xs
 fprimes
 :glyph-type 2
 :glyph-size 5
 :morelines (list (list 0 0 0 sorted-xs fs "ln"))
 :title "Adaptive Centered-Difference Approximations"
 :ytics (rseq -1 1 9)
 :ymin -1
 :ymax 1
 :x-axis t
 :y-axis t
 )

;;Let's go...
;; Here's our function:
(func f(x) sin(1/x))
(setq
 a .1
 b 1 
 )

;; initialize our global for plotting (the boundaries of each panel):

;; run the code:
(setq
 results (adaptive-centered-recursive #'f a b :epsilon 1e-2 :tolerance 1e-4)
 xs (mapcar #'first results)
 fprimes (mapcar #'second results)
 sorted-xs (sort-data xs)
 fs (mapcar #'f sorted-xs)
 done t
 )

;; Now plot it...
(xgobips 
 "AdaptiveDerivatives.part2.ps"
 0
 0
 1
 xs
 fprimes
 :glyph-type 2
 :glyph-size 5
 :morelines (list (list 0 0 0 sorted-xs fs "ln"))
 :title "Adaptive Centered-Difference Approximations"
 :x-axis t
 :y-axis t
 )
